#include <zephyr/types.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
#include <inttypes.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <zephyr/sys/printk.h>
#include <zephyr/sys/byteorder.h>
#include <zephyr/kernel.h>

#define PI          3.1415926535897932384626433832795
#define CHANNEL    3

//Highpass Filter
float prevHPInput[CHANNEL];
float prevHPOutput[CHANNEL];
float alpha_HP[CHANNEL];

// Lowpass Filter
float prevLPOutput[CHANNEL];
float alpha_LP[CHANNEL];

void begin_KickFiltersRT(void) {
    for (uint8_t i = 0 ; i<CHANNEL; i++) {
        // High Pass Filter
        prevHPInput[i] = 0;
        prevHPOutput[i] = 0;
        alpha_HP[i] = 0;

        // Low Pass Filter
        prevLPOutput[i] = 0;
        alpha_LP[i] = 0;
    }
}

//int16_t KickFiltersRT::highpass(const int16_t input, float fc, uint16_t dt)
//return		output of the filter
//
//input			input data
//fc			desired cut-off frequency of the filter (in Hertz)
//fs			sampling frequency (time in between samples) in Hertz (Hz)
//
//Implements a simple first-order high pass filter according to the algorithm
//here: https://en.wikipedia.org/wiki/High-pass_filter
//The Wikipedia article is also located in the library in the "extras/references/" folder.
float highpassA(const float input, float fc, float fs, int object) {
	//Tau = Resistance(R)*Capacitance(C)
	//re-arranging the cut-off frequency equaion [1/(2*pi*R*C)] to solve for R*C
	float tau = 1/(2.0*PI*fc);
	alpha_HP[object] = tau / (tau + (1/fs));
	
	//filtering function
	float output = alpha_HP[object]*(prevHPOutput[object] + input - prevHPInput[object]);
	
	//re-setting member variables for the next time the filter is called
	prevHPInput[object] = input;
	prevHPOutput[object] = output;
	
	return output;
}

//int16_t KickFiltersRT::highpass(const int16_t input)
//return		output of the filter
//
//input			input data
//
//This function only works if the filter has already been initialized
//i.e. fc, dt, alpha_HP, prevHPInput, and prevHPOutput have already been defined.
//
//Implements a simple first-order high pass filter according to the algorithm
//here: https://en.wikipedia.org/wiki/High-pass_filter
//The Wikipedia article is also located in the library in the "extras/references/" folder.
float highpassB(const float input, int object) {
	//filtering function
	float output = alpha_HP[object]*(prevHPOutput[object] + input - prevHPInput[object]);
	
	//re-setting member variables for the next time the filter is called
	prevHPInput[object] = input;
	prevHPOutput[object] = output;
	
	return output;
}

//void KickFiltersRT::inithighpass(const int16_t input0, float fc, uint16_t dt)
//
//input0		First value of input data. Feel free to set to 0.
//fc			desired cut-off frequency of the filter (in Hertz)
//fs			sampling frequency (time in between samples) in Hertz (Hz)
//
//Initializes filter by defining necessary variables.
//
//Implements a simple first-order high pass filter according to the algorithm
//here: https://en.wikipedia.org/wiki/High-pass_filter
//The Wikipedia article is also located in the library in the "extras/references/" folder.
void inithighpass(const float input0, float fc, float fs, int object)
{
	//Tau = Resistance(R)*Capacitance(C)
	//re-arranging the cut-off frequency equaion [1/(2*pi*R*C)] to solve for R*C
	float tau = 1/(2.0*PI*fc);
	alpha_HP[object] = tau / (tau + (1/fs));
	
	
	//setting member variables for the next time the filter is called
	prevHPInput[object] = input0;
	prevHPOutput[object] = input0;
}

//int16_t KickFiltersRT::lowpass(const int16_t input, float fc, uint16_t dt)
//return		output of the filter
//
//input			input data
//fc			desired cut-off frequency of the filter (in Hertz)
//fs			sampling frequency (time between samples) in Hertz (Hz)
//
//Implements a simple first-order low pass filter according to the algorithm
//here: https://en.wikipedia.org/wiki/Low-pass_filter
//The Wikipedia article is also located in the library in the "extras/references/" folder.
float lowpassA(const float input, float fc, float fs, int object)
{
	//Tau = Resistance*Capacitance
	//re-arranging the cut-off frequency equaion [1/(2*pi*R*C)] to solve for R*C
	float tau = 1/(2.0*PI*fc);
	alpha_LP[object] = (1/fs) / (tau + (1/fs));
	
	//filtering function
	float output = prevLPOutput[object] + alpha_LP[object]*(input - prevLPOutput[object]);
	
	//re-setting member variables for the next time the filter is called
	prevLPOutput[object] = output;
	
	return output;
}

//int16_t KickFiltersRT::lowpass(const int16_t input)
//return		output of the filter
//
//input			input data
//
//This function only works if the filter has already been initialized
//i.e. fc, dt, alpha_LP, and prevLPOutput have already been defined.
//
//Implements a simple first-order low pass filter according to the algorithm
//here: https://en.wikipedia.org/wiki/Low-pass_filter
//The Wikipedia article is also located in the library in the "extras/references/" folder.
float lowpassB(const float input, int object)
{
	//filtering function
	float output = prevLPOutput[object] + alpha_LP[object]*(input - prevLPOutput[object]);
	
	//re-setting member variables for the next time the filter is called
	prevLPOutput[object] = output;
	
	return output;
}

//void KickFiltersRT::initlowpass(const int16_t input0, float fc, uint16_t dt)
//
//input0		First value of input data. Feel free to set to 0.
//fc			desired cut-off frequency of the filter (in Hertz)
//fs			sampling frequency (time between samples) in Hertz (Hz)
//
//Initializes filter by defining necessary variables.
//
//Implements a simple first-order low pass filter according to the algorithm
//here: https://en.wikipedia.org/wiki/Low-pass_filter
//The Wikipedia article is also located in the library in the "extras/references/" folder.
void initlowpass(const float input0, float fc, float fs, int object)
{
	//Tau = Resistance*Capacitance
	//re-arranging the cut-off frequency equaion [1/(2*pi*R*C)] to solve for R*C
	float tau = 1/(2.0*PI*fc);
	alpha_LP[object] = (1/fs) / (tau + (1/fs));
	
	
	//setting member variables for the next time the filter is called
	prevLPOutput[object] = alpha_LP[object]*input0;
}